@@ -4,14 +4,13 @@ import os
 import logging
 
 import tensorflow as tf
-from tensorflow.python.keras.engine import training
 import tensorflow_addons as tfa
 import pandas as pd
 
-from ..data import NLPDataset
-from ..data.text_segmenter import get_segmenter
-from ..vocab import Vocab
-from ..callbacks import WeightDecayScheduler
+from sknlp.data import NLPDataset
+from sknlp.data.text_segmenter import get_segmenter
+from sknlp.vocab import Vocab
+from sknlp.callbacks import WeightDecayScheduler
 from .text2vec import Word2vec, Text2vec
 from .base_model import BaseNLPModel
 
@@ -37,18 +36,16 @@ class SupervisedNLPModel(BaseNLPModel):
         super().__init__(
             max_sequence_length=max_sequence_length,
             sequence_length=sequence_length,
-            **kwargs
+            segmenter=text2vec.segmenter if text2vec else segmenter,
         )
         self._text2vec = text2vec
         if text2vec is not None:
-            self._segmenter = text2vec.segmenter
             self._embedding_size = text2vec.embedding_size
             self._max_sequence_length = (
                 max_sequence_length or text2vec.max_sequence_length
             )
             self._sequence_length = text2vec.sequence_length
         else:
-            self._segmenter = segmenter
             self._embedding_size = embedding_size
             self._max_sequence_length = max_sequence_length
             self._sequence_length = sequence_length
@@ -62,10 +59,6 @@ class SupervisedNLPModel(BaseNLPModel):
     def embedding_size(self) -> int:
         return self._embedding_size
 
-    @property
-    def segmenter(self) -> str:
-        return self._segmenter
-
     @property
     def num_classes(self) -> int:
         return len(self._class2idx)
@@ -74,6 +67,16 @@ class SupervisedNLPModel(BaseNLPModel):
     def classes(self) -> List[str]:
         return list(self._class2idx.keys())
 
+    @property
+    def text2vec(self) -> Text2vec:
+        return self._text2vec
+
+    def class2idx(self, class_name: str) -> int:
+        return self._class2idx.get(class_name, None)
+
+    def idx2class(self, class_idx: int) -> str:
+        return self._idx2class.get(class_idx, None)
+
     def get_inputs(self) -> tf.Tensor:
         return self._text2vec.get_inputs()
 
@@ -109,13 +112,18 @@ class SupervisedNLPModel(BaseNLPModel):
             self._text2vec = Word2vec(vocab, self._embedding_size, self._segmenter)
 
         if dataset is not None:
-            return dataset.batchify(batch_size, shuffle=shuffle)
+            if isinstance(dataset, tf.data.Dataset):
+                return dataset
+            elif isinstance(dataset, NLPDataset):
+                return dataset.batchify(batch_size, shuffle=shuffle)
+            else:
+                raise ValueError("Unknown dataset.")
 
         if isinstance(y[0], (list, tuple)):
             y = ["|".join(map(str, y_i)) for y_i in y]
         df = pd.DataFrame(zip(X, y), columns=["text", "label"])
         dataset = self.create_dataset_from_df(
-            df, self._text2vec.vocab, self._text2vec.segmenter, self.classes
+            df, self.text2vec.vocab, self.text2vec.segmenter, self.classes
         )
         return dataset.batchify(
             batch_size, shuffle=shuffle, shuffle_buffer_size=df.shape[0]
@@ -137,11 +145,12 @@ class SupervisedNLPModel(BaseNLPModel):
         weight_decay: float = 1e-4,
         lr_update_factor: float = 0.5,
         lr_update_epochs: int = 10,
-        clip: float = 5.0,
+        clip: Optional[float] = 5.0,
         enable_early_stopping: bool = False,
         early_stopping_patience: Optional[int] = None,
         early_stopping_min_delta: float = 0,
         early_stopping_use_best_epoch: bool = False,
+        early_stopping_monitor: int = 2,  # 1 for loss, 2 for metric
         checkpoint: Optional[str] = None,
         save_frequency: int = 1,
         log_file: Optional[str] = None,
@@ -167,14 +176,14 @@ class SupervisedNLPModel(BaseNLPModel):
                 batch_size,
                 shuffle=False,
             )
+            self.valid_X = valid_X
+            self.valid_y = valid_y
         if not self._built:
             self.build()
         optimizer = tfa.optimizers.AdamW(weight_decay, learning_rate=lr, clipnorm=clip)
-        if optimizer != "adam":
-            pass
         loss = self.get_loss()
         metrics = self.get_metrics()
-        callbacks = self.get_callbacks()
+        callbacks = self.get_callbacks(batch_size)
         self._model.compile(optimizer=optimizer, loss=loss, metrics=metrics)
 
         def lr_scheduler(epoch, lr):
@@ -189,12 +198,18 @@ class SupervisedNLPModel(BaseNLPModel):
         weight_decay = WeightDecayScheduler(wd_scheduler, verbose=verbose)
         callbacks = [*callbacks, lr_decay, weight_decay]
         if enable_early_stopping:
+            if early_stopping_monitor == 2 and self.get_monitor():
+                mode = "max"
+                monitor = self.get_monitor()
+            else:
+                mode = "min"
+                monitor = "val_loss"
             callbacks.append(
                 tf.keras.callbacks.EarlyStopping(
-                    monitor=self.get_monitor(),
+                    monitor=monitor,
                     min_delta=early_stopping_min_delta,
                     patience=early_stopping_patience or lr_update_epochs,
-                    mode="max",
+                    mode=mode,
                     restore_best_weights=early_stopping_use_best_epoch,
                 )
             )
@@ -237,7 +252,6 @@ class SupervisedNLPModel(BaseNLPModel):
     def get_config(self) -> Dict[str, Any]:
         return {
             **super().get_config(),
-            "segmenter": self._segmenter,
             "classes": self.classes,
             "task": self._task,
             "algorithm": self._algorithm,
@@ -245,20 +259,25 @@ class SupervisedNLPModel(BaseNLPModel):
 
     def save(self, directory: str) -> None:
         super().save(directory)
-        self._text2vec.save_vocab(directory)
+        self.text2vec.save_vocab(directory)
 
     @classmethod
     def load(cls, directory: str) -> "SupervisedNLPModel":
         module = super().load(directory)
         with open(os.path.join(directory, "vocab.json")) as f:
             vocab = Vocab.from_json(f.read())
-        module._text2vec = Text2vec(vocab)
+        module._text2vec = Text2vec(
+            vocab,
+            segmenter=module.segmenter,
+            max_sequence_length=module.max_sequence_length,
+            sequence_length=module.sequence_length,
+        )
         return module
 
     def export(self, directory: str, name: str, version: str = "0") -> None:
         super().export(directory, name, version)
         d = os.path.join(directory, name, version)
-        self._text2vec.save_vocab(d)
+        self.text2vec.save_vocab(d)
 
     def get_custom_objects(self) -> Dict[str, Any]:
         return {**super().get_custom_objects(), "AdamW": tfa.optimizers.AdamW}
