@@ -1,4 +1,5 @@
-from typing import Sequence, List, Dict, Any, Optional, Union
+from typing import Sequence, List, Dict, Any, Optional
+from functools import partial
 
 import numpy as np
 import pandas as pd
@@ -7,7 +8,7 @@ import tensorflow as tf
 
 from sknlp.vocab import Vocab
 from sknlp.data import TaggingDataset
-from sknlp.metrics import PrecisionWithLogits, RecallWithLogits
+from sknlp.callbacks import ModelScoreCallback
 
 from ..supervised_model import SupervisedNLPModel
 from ..text2vec import Text2vec
@@ -52,11 +53,29 @@ class DeepTagger(SupervisedNLPModel):
         )
         self._loss = loss
 
+    @property
+    def start_tag(self) -> str:
+        return self._start_tag
+
+    @property
+    def end_tag(self) -> str:
+        return self._end_tag
+
+    @property
+    def pad_tag(self) -> str:
+        return self._pad_tag
+
     def get_loss(self) -> None:
         return None
 
-    def get_callbacks(self) -> List[tf.keras.callbacks.Callback]:
-        return []
+    def get_callbacks(self, batch_size: int) -> List[tf.keras.callbacks.Callback]:
+        return [
+            ModelScoreCallback(
+                partial(self.score, batch_size=batch_size),
+                self.valid_X,
+                self.valid_y
+            )
+        ]
 
     def get_metrics(self) -> List[tf.keras.metrics.Metric]:
         return []
@@ -71,10 +90,10 @@ class DeepTagger(SupervisedNLPModel):
             vocab,
             list(labels),
             df=df,
-            max_length=self._max_sequence_length,
+            max_length=self.max_sequence_length,
             text_segmenter=segmenter,
-            start_tag=self._start_tag,
-            end_tag=self._end_tag,
+            start_tag=self.start_tag,
+            end_tag=self.end_tag,
         )
 
     def dummy_y(self, X: Sequence[str]) -> List[List[str]]:
@@ -88,15 +107,15 @@ class DeepTagger(SupervisedNLPModel):
         batch_size: int = 128
     ) -> List[List[str]]:
         tag_ids_list = super().predict(X, dataset=dataset, batch_size=batch_size)
-        exclude_tags = {self._pad_tag, self._start_tag, self._end_tag}
+        exclude_tags = {self.pad_tag, self.start_tag, self.end_tag}
         predictions = []
         for tag_ids in tag_ids_list:
             tags = []
             for tag_id in tag_ids:
-                tag = self._idx2class[tag_id]
+                tag = self.idx2class(tag_id)
                 if tag not in exclude_tags:
                     tags.append(tag)
-                if tag == self._pad_tag:
+                if tag == self.pad_tag:
                     break
             predictions.append(tags)
         return predictions
@@ -105,12 +124,10 @@ class DeepTagger(SupervisedNLPModel):
         self,
         X: Sequence[str] = None,
         y: Sequence[Sequence[str]] = None,
-        *,
-        dataset: TaggingDataset = None,
         batch_size: int = 128
     ) -> pd.DataFrame:
-        predictions = self.predict(X=X, dataset=dataset, batch_size=batch_size)
-        labels = [c.split("-")[-1] for c in self.classes if "-" in c]
+        predictions = self.predict(X=X, batch_size=batch_size)
+        labels = list(set([c.split("-")[-1] for c in self.classes if "-" in c]))
         return tagging_fscore(X, y, predictions, labels)
 
     def format_score(self, score_df: pd.DataFrame, format: str = "markdown") -> str:
@@ -128,11 +145,4 @@ class DeepTagger(SupervisedNLPModel):
         config = super()._filter_config(config)
         config.pop("algorithm", None)
         config.pop("task", None)
-        return config
-
-    def get_custom_objects(self) -> Dict[str, Any]:
-        return {
-            **super().get_custom_objects(),
-            "PrecisionWithLogits": PrecisionWithLogits,
-            "RecallWithLogits": RecallWithLogits,
-        }
\ No newline at end of file
+        return config
\ No newline at end of file
